/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2020-2023, NVIDIA CORPORATION & AFFILIATES. All rights reserved.
 */

/*
 * Function naming determines intended use:
 *
 *     <x>_r(void) : Returns the offset for register <x>.
 *
 *     <x>_o(void) : Returns the offset for element <x>.
 *
 *     <x>_w(void) : Returns the word offset for word (4 byte) element <x>.
 *
 *     <x>_<y>_s(void) : Returns size of field <y> of register <x> in bits.
 *
 *     <x>_<y>_f(u32 v) : Returns a value based on 'v' which has been shifted
 *         and masked to place it at field <y> of register <x>.  This value
 *         can be |'d with others to produce a full register value for
 *         register <x>.
 *
 *     <x>_<y>_m(void) : Returns a mask for field <y> of register <x>.  This
 *         value can be ~'d and then &'d to clear the value of field <y> for
 *         register <x>.
 *
 *     <x>_<y>_<z>_f(void) : Returns the constant value <z> after being shifted
 *         to place it at field <y> of register <x>.  This value can be |'d
 *         with others to produce a full register value for <x>.
 *
 *     <x>_<y>_v(u32 r) : Returns the value of field <y> from a full register
 *         <x> value 'r' after being shifted to place its LSB at bit 0.
 *         This value is suitable for direct comparison with other unshifted
 *         values appropriate for use in field <y> of register <x>.
 *
 *     <x>_<y>_<z>_v(void) : Returns the constant value for <z> defined for
 *         field <y> of register <x>.  This value is suitable for direct
 *         comparison with unshifted values appropriate for use in field <y>
 *         of register <x>.
 */
#ifndef HW_AON_H
#define HW_AON_H

static inline u32 pm_r5_ctrl_r(void)
{
	return 0x1f0040U;
}
static inline u32 pm_r5_ctrl_fwloaddone_halted_f(void)
{
	return 0x0U;
}
static inline u32 pm_r5_ctrl_fwloaddone_done_f(void)
{
	return 0x2U;
}

static inline u32 evp_reset_addr_r(void)
{
	return 0x20U;
}

static inline u32 hsp_ss_base_r(void)
{
	return 0x1a0000U;
}

static inline u32 hsp_sm_base_r(void)
{
	return 0x160000U;
}

static inline u32 ast_ast0_base_r(void)
{
	return 0x40000U;
}

static inline u32 ast_ast1_base_r(void)
{
	return 0x50000U;
}

#endif
